"""hotfix_cestoreuploader

Due to recent Windows security updates, the CeStore Uploader functionality is sometimes broken,
resulting in failed uploads.

This hotfix resolves the issue, and will be included in BN 7.6.3.
"""

#//$autorun;event=PostCreateMainWin;

import bns
import ctypes
import json
from _multiprocessing import win32
try:
	from wgMLST_Client.SchedulerCommonTools import CeStore
except ImportError:
	# wgMLST_Client plugin not installed...
	pass
else:
	latestVersionToFix = '7.6.3894.32287'
	currentVersion = bns.Database.Db.Info.SoftwareVersionDetail[:len(latestVersionToFix)] # strip off platform information
	if currentVersion <= latestVersionToFix:
		# otherwise, already fixed in release
		
		def test(namedpipe):
			try:
				path = r'\\.\pipe\{0}'.format(namedpipe.name)
				win32.WaitNamedPipe(path, 100)
				return True
			except WindowsError as e:
				bns.Util.Program.AddProgramLog('WaitNamedPipe failed: {0}'.format(e))
				return False
		
		def open(namedpipe):
			path = r'\\.\pipe\{0}'.format(namedpipe.name)
			namedpipe.fh = win32.CreateFile(path, win32.GENERIC_WRITE, 0, win32.NULL, win32.OPEN_EXISTING, 0, win32.NULL)
		
		def close(namedpipe):
			ctypes.windll.kernel32.FlushFileBuffers(namedpipe.fh)
			win32.CloseHandle(namedpipe.fh)
		
		def write_json(namedpipe, data):
			written = ctypes.c_ulong(0)
			data_str = json.dumps(data)
			data_len = len(data_str)
			while data_len > 0:
				write_ok = ctypes.windll.kernel32.WriteFile(namedpipe.fh, ctypes.cast(data_str, ctypes.c_void_p), data_len, ctypes.byref(written), None)
				if not write_ok:
					err = ctypes.WinError()
					namedpipe.close()
					raise err
				if written.value >= data_len:
					break
				data_str = data_str[written.value:]
				data_len = len(data_str)
		
		CeStore.NamedPipe.test = test
		CeStore.NamedPipe.open = open
		CeStore.NamedPipe.close = close
		CeStore.NamedPipe.write_json = write_json
